﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Reactive.Linq;
using System.Reactive.Concurrency;
using System.Threading;

namespace RxVSMSample.ConsoleSequence
{
    class Program
    {
        static void Main(string[] args)
        {
            List<ulong> fibonacciSeq = GenerateFibonnaci(1000);
            IObservable<ulong> fibonacciSource = fibonacciSeq.ToObservable();

            Console.WriteLine("Press enter to subscribe");
            Console.ReadLine();

            using (var fibonnacciSub = fibonacciSource.SubscribeOn(Scheduler.TaskPool)
                .Subscribe(
                (x) => Console.WriteLine(x),
                (Exception ex) => Console.WriteLine("Error received from source: {0}.", ex.Message),
                () => Console.WriteLine("End of sequence.")
                )
            )
            {
                Console.WriteLine("Press enter again to unsubscribe from the service.");
                Console.ReadLine();
            }
        }

        static List<ulong> GenerateFibonnaci(int sequenceLength)
        {
            List<ulong> sequence = new List<ulong>();

            for (int i = 0; i < sequenceLength; i++)
            {
                if (i <= 1)
                    sequence.Add((ulong)i);
                else
                    sequence.Add(sequence[i - 2] + sequence[i - 1]);
            }

            return sequence;
        }
    }
}
